@@ -24,6 +24,7 @@ PyTypeObject GraphicsManager_Type;
 
 #define DEFAULT_STORAGE_LIMIT 320u * (1024u * 1024u)
 #define REPORT_ERROR(...) { log_error(__VA_ARGS__); }
+#define FREE_CFD_AFTER_FUNCTION __attribute__((cleanup(cfd_free)))
 
 // caching {{{
 #define CACHE_KEY_BUFFER_SIZE 32
@@ -802,13 +803,6 @@ grman_update_layers(GraphicsManager *self, unsigned int scrolled_by, float scree
 
 // Animation {{{
 #define DEFAULT_GAP 40
-#define _frame_number num_lines
-#define _other_frame_number num_cells
-#define _gap z_index
-#define _animation_state data_width
-#define _blend_mode cell_x_offset
-#define _bgcolor cell_y_offset
-#define _loop_count data_height
 
 static inline Frame*
 current_frame(Image *img) {
@@ -825,7 +819,7 @@ frame_for_id(Image *img, const uint32_t frame_id) {
     return NULL;
 }
 
-static inline Frame*
+static Frame*
 frame_for_number(Image *img, const uint32_t frame_number) {
     switch(frame_number) {
         case 1:
@@ -872,10 +866,53 @@ alpha_blend(uint8_t *dest_px, const uint8_t *src_px) {
 typedef struct {
     bool needs_blending;
     uint32_t over_px_sz, under_px_sz;
-    uint32_t over_width, over_height, under_width, under_height, over_offset_x, over_offset_y;
+    uint32_t over_width, over_height, under_width, under_height, over_offset_x, over_offset_y, under_offset_x, under_offset_y;
+    uint32_t stride;
 } ComposeData;
 
-static inline void
+#define COPY_RGB under_px[0] = over_px[0]; under_px[1] = over_px[1]; under_px[2] = over_px[2];
+#define COPY_PIXELS \
+    if (d.needs_blending) { \
+        if (d.under_px_sz == 3) { \
+            ROW_ITER PIX_ITER blend_on_opaque(under_px, over_px); }} \
+        } else { \
+            ROW_ITER PIX_ITER alpha_blend(under_px, over_px); }} \
+        } \
+    } else { \
+        if (d.under_px_sz == 4) { \
+            if (d.over_px_sz == 4) { \
+                ROW_ITER PIX_ITER COPY_RGB under_px[3] = over_px[3]; }} \
+            } else { \
+                ROW_ITER PIX_ITER COPY_RGB under_px[3] = 255; }} \
+            } \
+        } else { \
+            ROW_ITER PIX_ITER COPY_RGB }} \
+        } \
+    } \
+
+
+static void
+compose_rectangles(const ComposeData d, uint8_t *under_data, const uint8_t *over_data) {
+    // compose two equal sized, non-overlapping rectangles at different offsets
+    // does not do bounds checking on the data arrays
+    const bool can_copy_rows = !d.needs_blending && d.over_px_sz == d.under_px_sz;
+    const unsigned min_width = MIN(d.under_width, d.over_width);
+#define ROW_ITER for (unsigned y = 0; y < d.under_height && y < d.over_height; y++) { \
+        uint8_t *under_row = under_data + (y + d.under_offset_y) * d.under_px_sz * d.stride + (d.under_offset_x * d.under_px_sz); \
+        const uint8_t *over_row = over_data + (y + d.over_offset_y) * d.over_px_sz * d.stride + (d.over_offset_x * d.over_px_sz);
+    if (can_copy_rows) {
+        ROW_ITER memcpy(under_row, over_row, (size_t)d.over_px_sz * min_width);}
+        return;
+    }
+#define PIX_ITER for (unsigned x = 0; x < min_width; x++) { \
+        uint8_t *under_px = under_row + (d.under_px_sz * x); \
+        const uint8_t *over_px = over_row + (d.over_px_sz * x);
+    COPY_PIXELS
+#undef PIX_ITER
+#undef ROW_ITER
+}
+
+static void
 compose(const ComposeData d, uint8_t *under_data, const uint8_t *over_data) {
     const bool can_copy_rows = !d.needs_blending && d.over_px_sz == d.under_px_sz;
     unsigned min_row_sz = d.over_offset_x < d.under_width ? d.under_width - d.over_offset_x : 0;
@@ -890,24 +927,7 @@ compose(const ComposeData d, uint8_t *under_data, const uint8_t *over_data) {
 #define PIX_ITER for (unsigned x = 0; x < min_row_sz; x++) { \
         uint8_t *under_px = under_row + (d.under_px_sz * x); \
         const uint8_t *over_px = over_row + (d.over_px_sz * x);
-#define COPY_RGB under_px[0] = over_px[0]; under_px[1] = over_px[1]; under_px[2] = over_px[2];
-    if (d.needs_blending) {
-        if (d.under_px_sz == 3) {
-            ROW_ITER PIX_ITER blend_on_opaque(under_px, over_px); }}
-        } else {
-            ROW_ITER PIX_ITER alpha_blend(under_px, over_px); }}
-        }
-    } else {
-        if (d.under_px_sz == 4) {
-            if (d.over_px_sz == 4) {
-                ROW_ITER PIX_ITER COPY_RGB under_px[3] = over_px[3]; }}
-            } else {
-                ROW_ITER PIX_ITER COPY_RGB under_px[3] = 255; }}
-            }
-        } else {
-            ROW_ITER PIX_ITER COPY_RGB }}
-        }
-    }
+    COPY_PIXELS
 #undef COPY_RGB
 #undef PIX_ITER
 #undef ROW_ITER
@@ -982,7 +1002,7 @@ get_coalesced_frame_data_impl(GraphicsManager *self, Image *img, const Frame *f,
     return base_data;
 }
 
-static inline CoalescedFrameData
+static CoalescedFrameData
 get_coalesced_frame_data(GraphicsManager *self, Image *img, const Frame *f) {
     return get_coalesced_frame_data_impl(self, img, f, 0);
 }
@@ -1021,10 +1041,10 @@ reference_chain_too_large(Image *img, const Frame *frame) {
 
 static Image*
 handle_animation_frame_load_command(GraphicsManager *self, GraphicsCommand *g, Image *img, const uint8_t *payload, bool *is_dirty) {
-    uint32_t frame_number = g->_frame_number, fmt = g->format ? g->format : RGBA;
+    uint32_t frame_number = g->frame_number, fmt = g->format ? g->format : RGBA;
     if (!frame_number || frame_number > img->extra_framecnt + 2) frame_number = img->extra_framecnt + 2;
     bool is_new_frame = frame_number == img->extra_framecnt + 2;
-    g->_frame_number = frame_number;
+    g->frame_number = frame_number;
     unsigned char tt = g->transmission_type ? g->transmission_type : 'd';
     if (tt == 'd' && self->currently_loading.loading_for.image_id == img->internal_id) {
         INIT_CHUNKED_LOAD;
@@ -1058,9 +1078,9 @@ handle_animation_frame_load_command(GraphicsManager *self, GraphicsCommand *g, I
         .x = g->x_offset, .y = g->y_offset,
         .is_4byte_aligned = load_data->is_4byte_aligned,
         .is_opaque = load_data->is_opaque,
-        .alpha_blend = g->_blend_mode != 1 && !load_data->is_opaque,
-        .gap = g->_gap > 0 ? g->_gap : (g->_gap < 0) ? 0 : DEFAULT_GAP,
-        .bgcolor = g->_bgcolor,
+        .alpha_blend = g->blend_mode != 1 && !load_data->is_opaque,
+        .gap = g->gap > 0 ? g->gap : (g->gap < 0) ? 0 : DEFAULT_GAP,
+        .bgcolor = g->bgcolor,
     };
     Frame *frame;
     if (is_new_frame) {
@@ -1071,11 +1091,11 @@ handle_animation_frame_load_command(GraphicsManager *self, GraphicsCommand *g, I
         img->extra_framecnt++;
         frame = img->extra_frames + frame_number - 2;
         const ImageAndFrame key = { .image_id = img->internal_id, .frame_id = transmitted_frame.id };
-        if (g->_other_frame_number) {
-            Frame *other_frame = frame_for_number(img, g->_other_frame_number);
+        if (g->other_frame_number) {
+            Frame *other_frame = frame_for_number(img, g->other_frame_number);
             if (!other_frame) {
                 img->extra_framecnt--;
-                ABRT("EINVAL", "No frame with number: %u found", g->_other_frame_number);
+                ABRT("EINVAL", "No frame with number: %u found", g->other_frame_number);
             }
             if (other_frame->base_frame_id && reference_chain_too_large(img, other_frame)) {
                 // since there is a long reference chain to render this frame, make
@@ -1114,7 +1134,7 @@ handle_animation_frame_load_command(GraphicsManager *self, GraphicsCommand *g, I
     } else {
         frame = frame_for_number(img, frame_number);
         if (!frame) ABRT("EINVAL", "No frame with number: %u found", frame_number);
-        if (g->_gap != 0) change_gap(img, frame, transmitted_frame.gap);
+        if (g->gap != 0) change_gap(img, frame, transmitted_frame.gap);
         CoalescedFrameData cfd = get_coalesced_frame_data(self, img, frame);
         if (!cfd.buf) ABRT("EINVAL", "No data associated with frame number: %u", frame_number);
         frame->alpha_blend = false; frame->base_frame_id = 0; frame->bgcolor = 0;
@@ -1157,7 +1177,7 @@ handle_delete_frame_command(GraphicsManager *self, const GraphicsCommand *g, boo
         REPORT_ERROR("Animation command refers to non-existent image with id: %u and number: %u", g->id, g->image_number);
         return NULL;
     }
-    uint32_t frame_number = MIN(img->extra_framecnt + 1, g->_frame_number);
+    uint32_t frame_number = MIN(img->extra_framecnt + 1, g->frame_number);
     if (!frame_number) frame_number = 1;
     if (!img->extra_framecnt) return g->delete_action == 'F' ? img : NULL;
     *is_dirty = true;
@@ -1193,24 +1213,24 @@ handle_delete_frame_command(GraphicsManager *self, const GraphicsCommand *g, boo
 
 static void
 handle_animation_control_command(GraphicsManager *self, bool *is_dirty, const GraphicsCommand *g, Image *img) {
-    if (g->_frame_number) {
-        uint32_t frame_idx = g->_frame_number - 1;
+    if (g->frame_number) {
+        uint32_t frame_idx = g->frame_number - 1;
         if (frame_idx <= img->extra_framecnt) {
             Frame *f = frame_idx ? img->extra_frames + frame_idx - 1 : &img->root_frame;
-            if (g->_gap) change_gap(img, f, g->_gap);
+            if (g->gap) change_gap(img, f, g->gap);
         }
     }
-    if (g->_other_frame_number) {
-        uint32_t frame_idx = g->_other_frame_number - 1;
+    if (g->other_frame_number) {
+        uint32_t frame_idx = g->other_frame_number - 1;
         if (frame_idx != img->current_frame_index && frame_idx <= img->extra_framecnt) {
             img->current_frame_index = frame_idx;
             *is_dirty = true;
             update_current_frame(self, img, NULL);
         }
     }
-    if (g->_animation_state) {
+    if (g->animation_state) {
         AnimationState old_state = img->animation_state;
-        switch(g->_animation_state) {
+        switch(g->animation_state) {
             case 1:
                 img->animation_state = ANIMATION_STOPPED; break;
             case 2:
@@ -1229,8 +1249,8 @@ handle_animation_control_command(GraphicsManager *self, bool *is_dirty, const Gr
         }
         img->current_loop = 0;
     }
-    if (g->_loop_count) {
-        img->max_loops = g->_loop_count - 1;
+    if (g->loop_count) {
+        img->max_loops = g->loop_count - 1;
         global_state.check_for_active_animated_images = true;
     }
 }
@@ -1278,6 +1298,73 @@ scan_active_animations(GraphicsManager *self, const monotonic_t now, monotonic_t
 }
 // }}}
 
+// {{{ composition a=c
+static void
+cfd_free(void *p) { free(((CoalescedFrameData*)p)->buf); }
+
+static void
+handle_compose_command(GraphicsManager *self, bool *is_dirty, const GraphicsCommand *g, Image *img) {
+    Frame *src_frame = frame_for_number(img, g->frame_number);
+    if (!src_frame) {
+        set_command_failed_response("ENOENT", "No source frame number %u exists in image id: %u\n", g->frame_number, img->client_id);
+        return;
+    }
+    Frame *dest_frame = frame_for_number(img, g->other_frame_number);
+    if (!dest_frame) {
+        set_command_failed_response("ENOENT", "No destination frame number %u exists in image id: %u\n", g->other_frame_number, img->client_id);
+        return;
+    }
+    const unsigned int width = g->width ? g->width : img->width;
+    const unsigned int height = g->height ? g->height : img->height;
+    const unsigned int dest_x = g->x_offset, dest_y = g->y_offset, src_x = g->cell_x_offset, src_y = g->cell_y_offset;
+    if (dest_x + width > img->width || dest_y + height > img->height) {
+        set_command_failed_response("EINVAL", "The destination rectangle is out of bounds");
+        return;
+    }
+    if (src_x + width > img->width || src_y + height > img->height) {
+        set_command_failed_response("EINVAL", "The source rectangle is out of bounds");
+        return;
+    }
+    if (src_frame == dest_frame) {
+        bool x_overlaps = MAX(src_x, dest_x) < (MIN(src_x, dest_x) + width);
+        bool y_overlaps = MAX(src_y, dest_y) < (MIN(src_y, dest_y) + height);
+        if (x_overlaps && y_overlaps) {
+            set_command_failed_response("EINVAL", "The source and destination rectangles overlap and the src and destination frames are the same");
+            return;
+        }
+    }
+
+    FREE_CFD_AFTER_FUNCTION CoalescedFrameData src_data = get_coalesced_frame_data(self, img, src_frame);
+    if (!src_data.buf) {
+        set_command_failed_response("EINVAL", "Failed to get data for src frame: %u", g->frame_number - 1);
+        return;
+    }
+    FREE_CFD_AFTER_FUNCTION CoalescedFrameData dest_data = get_coalesced_frame_data(self, img, dest_frame);
+    if (!dest_data.buf) {
+        set_command_failed_response("EINVAL", "Failed to get data for destination frame: %u", g->other_frame_number - 1);
+        return;
+    }
+    ComposeData d = {
+        .over_px_sz = src_data.is_opaque ? 3 : 4, .under_px_sz = dest_data.is_opaque ? 3: 4,
+        .needs_blending = !g->compose_mode && !src_data.is_opaque,
+        .over_offset_x = src_x, .over_offset_y = src_y,
+        .under_offset_x = dest_x, .under_offset_y = dest_y,
+        .over_width = width, .over_height = height, .under_width = width, .under_height = height,
+        .stride = img->width
+    };
+    compose_rectangles(d, dest_data.buf, src_data.buf);
+    const ImageAndFrame key = { .image_id = img->internal_id, .frame_id = dest_frame->id };
+    if (!add_to_cache(self, key, dest_data.buf, ((size_t)(dest_data.is_opaque ? 3 : 4)) * img->width * img->height)) {
+        if (PyErr_Occurred()) PyErr_Print();
+        set_command_failed_response("ENOSPC", "Failed to store image data in disk cache");
+    }
+    // frame is now a fully coalesced frame
+    dest_frame->x = 0; dest_frame->y = 0; dest_frame->width = img->width; dest_frame->height = img->height;
+    dest_frame->base_frame_id = 0; dest_frame->bgcolor = 0;
+    *is_dirty = (g->other_frame_number - 1) == img->current_frame_index;
+}
+// }}}
+
 // Image lifetime/scrolling {{{
 
 static inline void
@@ -1299,14 +1386,18 @@ filter_refs(GraphicsManager *self, const void* data, bool free_images, bool (*fi
 }
 
 
-static inline void
-modify_refs(GraphicsManager *self, const void* data, bool free_images, bool (*filter_func)(ImageRef*, Image*, const void*, CellPixelSize), CellPixelSize cell) {
+static void
+modify_refs(GraphicsManager *self, const void* data, bool (*filter_func)(ImageRef*, Image*, const void*, CellPixelSize), CellPixelSize cell) {
     for (size_t i = self->image_count; i-- > 0;) {
         Image *img = self->images + i;
         for (size_t j = img->refcnt; j-- > 0;) {
             if (filter_func(img->refs + j, img, data, cell)) remove_i_from_array(img->refs, j, img->refcnt);
         }
-        if (img->refcnt == 0 && (free_images || img->client_id == 0)) remove_image(self, i);
+        if (img->refcnt == 0 && img->client_id == 0 && img->client_number == 0) {
+            // references have all scrolled off the history buffer and the image has no way to reference it
+            // to create new references so remove it.
+            remove_image(self, i);
+        }
     }
 }
 
@@ -1363,7 +1454,7 @@ void
 grman_scroll_images(GraphicsManager *self, const ScrollData *data, CellPixelSize cell) {
     if (self->image_count) {
         self->layers_dirty = true;
-        modify_refs(self, data, true, data->has_margins ? scroll_filter_margins_func : scroll_filter_func, cell);
+        modify_refs(self, data, data->has_margins ? scroll_filter_margins_func : scroll_filter_func, cell);
     }
 }
 
@@ -1560,6 +1651,20 @@ grman_handle_command(GraphicsManager *self, const GraphicsCommand *g, const uint
         case 'd':
             handle_delete_command(self, g, c, is_dirty, cell);
             break;
+        case 'c':
+            if (!g->id && !g->image_number) {
+                REPORT_ERROR("Compose frame data command without image id or number");
+                break;
+            }
+            Image *img = g->id ? img_by_client_id(self, g->id) : img_by_client_number(self, g->image_number);
+            if (!img) {
+                set_command_failed_response("ENOENT", "Animation command refers to non-existent image with id: %u and number: %u", g->id, g->image_number);
+                ret = finish_command_response(g, false);
+            } else {
+                handle_compose_command(self, is_dirty, g, img);
+                ret = finish_command_response(g, true);
+            }
+            break;
         default:
             REPORT_ERROR("Unknown graphics command action: %c", g->action);
             break;
